--[[
	##AddOn	: CooldownIcons R(evamped)
	##Subfile 	: Options\icon_settings.lua
	##Author  : Hati ( xhatix on wowace.com )
--]]
local CIR = CooldownIconsR
local options = CIR.Options
local media = LibStub("LibSharedMedia-3.0")
local LBF = LibStub("LibButtonFacade", true)
local LCI = LibStub("LibCooldownIcons") -- this is more a personal utility lib (provides some 'extra' functions or extended standard ones)

local DftlValues = {
	iEnabled = true,
}

local function iconConfigHandler(obj,th,v)
	local icons = CIR.icons
	local db = CIR.db.char.icondb
	--General
	if th=="iEnabled" then
		if v and not icons[obj] then
			CIR:CreateIcon(obj)
		end
		icons[obj].active = v
		if v then
            icons[obj]:Show()
			icons[obj]:SetDisplay(false)
			CIR:ProcessScriptForIcon(db[obj].script,icons[obj])
			CIR:SPELL_UPDATE_COOLDOWN()
		else
			icons[obj]:Hide()
            CIR:UnregisterObjFromAllEvents(obj)
		end
	elseif th=='iZoom' then
		local mZoom = LCI:tonumber(v) *.08
		icons[obj].texture:SetTexCoord(0.0+mZoom, 1.0-mZoom, 0.0+mZoom, 1.0-mZoom)
	elseif th=='iRange' then
		if not v then
			icons[obj]:RangeColoringOff()
			local _, i = LCI:inTable(obj, CIR.rangeCheck)
			tremove(CIR.rangeCheck,i)
		else
			icons[obj]:ResetRange()
			CIR.rangeCheck[#CIR.rangeCheck+1] = obj
		end
	elseif th=='iGrayscale' then
		if not v then
			icons[obj]:GrayscaleOff()
		else
			icons[obj]:ResetGray()
		end
	elseif th=='iIconSize' then
		icons[obj]:SetWidth(v)
		icons[obj].texture:SetWidth(v)
		icons[obj]:SetHeight(v)
		icons[obj].texture:SetHeight(v)
		iconConfigHandler(obj,'iZoom',db[obj].iZoom)
        if icons[obj].glowFrame then
            icons[obj].glowFrame.innerGlow:SetPoint("TOPLEFT", icons[obj], "TOPLEFT", -v * 0.2, v * 0.2)
            icons[obj].glowFrame.innerGlow:SetPoint("BOTTOMRIGHT", icons[obj], "BOTTOMRIGHT", v * 0.2, -v*0.2)
            icons[obj].glowFrame.outerGlow:SetPoint("TOPLEFT", icons[obj], "TOPLEFT", -v * 0.2, v * 0.2)
            icons[obj].glowFrame.outerGlow:SetPoint("BOTTOMRIGHT", icons[obj], "BOTTOMRIGHT", v * 0.2, -v*0.2)
            icons[obj].glowFrame.ants:SetPoint("TOPLEFT", icons[obj], "TOPLEFT", -v * 0.2, v * 0.2)
            icons[obj].glowFrame.ants:SetPoint("BOTTOMRIGHT", icons[obj], "BOTTOMRIGHT", v * 0.2, -v*0.2)
        end
		if LBF then
			LBF:Group('CooldownIcons R'):Skin( unpack(CIR.db.char.BFStyle or { 'Blizzard', 0, nil }))
		end
	elseif th=='iX' then
		icons[obj]:SetPoint("CENTER",v,icons[obj]:GetY())
	elseif th=='iY' then
		icons[obj]:SetPoint("CENTER",icons[obj]:GetX(),v)
	elseif th=='script' then
		CIR:ProcessScriptForIcon(v,icons[obj])
	elseif th=='iDisplayName' then
		if v=="" then
			v=obj
		end
		CIR:add_icon_settings(obj,v)
	--Font
	elseif th=="iFontSize" then
		icons[obj].text:SetFont(media:Fetch("font",media:List('font')[db[obj].iFontStyle] or "Fonts\\FRIZQT__.TTF"),v,"OUTLINE")
--[[	elseif th=="iFontStyle" then
		print(v)
		icons[obj].text:SetFont(media:Fetch("font",media:List('font')[v] or "Fonts\\FRIZQT__.TTF"),db[obj].iFontSize,"OUTLINE")]]
	elseif th=="iFontOffsetX" then
		icons[obj].text:SetPoint("CENTER",icons[obj]:GetName(),"CENTER",v,db[obj].iFontOffsetY)
	elseif th=="iFontOffsetY" then
		icons[obj].text:SetPoint("CENTER",icons[obj]:GetName(),"CENTER",db[obj].iFontOffsetX,v)
	elseif th=="iFontColor" then
		icons[obj].text:SetTextColor(unpack(v or {1.0,1.0,1.0,1.0}))
	--cosmetic types
	elseif th=="iDisplayType" then
		icons[obj].displayType = v
		if v==1 then
			icons[obj]:SetAlpha(1)
		elseif v==2 then
            if tContains(CIR.OnCooldown,obj) then
                icons[obj]:SetDisplay(true)
            else
                icons[obj]:SetDisplay(false)
            end
		elseif v==3 then
            if tContains(CIR.OnCooldown,obj) then
                icons[obj]:SetDisplay(true)
            else
                icons[obj]:SetDisplay(false)
            end
		end
        local script = CIR.db.char.icondb[obj].script
		CIR:ProcessScriptForIcon(script,icons[obj])
	elseif th=="iTimeFormat" then
		icons[obj].timeFormat = v
		--Call request frame, to make individual input and not bloating the option frame
	end
end
CIR.iconConfigHandler = function(self,obj,th,v)
    iconConfigHandler(obj,th,v)
end

function CIR:getIconValue(info)
	--if already an value in icondb use it otherwise use DftlValues one
	return CIR.db.char.icondb[info[#info-2]][info[#info]] or DftlValues[info[#info-2]]
end
function CIR:setIconValue(info,value)
	local obj,th = info[#info-2],info[#info]
	CIR.db.char.icondb[obj][th] = value
	if CIR.icons[obj] then
		if not CIR.icons[obj].active and th=="iEnabled" then
			iconConfigHandler(obj,th,value)
		elseif CIR.icons[obj].active then
			iconConfigHandler(obj,th,value)
		end
	elseif not CIR.icons[obj] and th=="iEnabled" then
		iconConfigHandler(obj,th,value)
	end
end

local function remCurBtn_Func(info)
	local obj = info[#info-1]
	local icon
	if CIR.icons[obj] then
		icon = CIR.icons[obj]
	end
	icon:Hide()
	--icon:SetParent(nil)
	print("Removing Icon: ",info[#info-1])
	local inRC, i = LCI:inTable(info[#info-1], CIR.rangeCheck)
	if inRC then
		tremove(CIR.rangeCheck,i)
	end
	CIR.Options.args.icon_settings.args = LCI:deepcopyWithoutElement(CIR.Options.args.icon_settings.args,info[#info-1])
	CIR.db.char.icondb=LCI:deepcopyWithoutElement(CIR.db.char.icondb,info[#info-1])
	for k,v in pairs(CIR.icons) do
		v:SetParent(nil)
		v:Hide()
	end
	table.wipe(CIR.icons)
	for k,v in pairs(CIR.db.char.icondb) do
		--print("["..k.."] = "..tostring(v))
		CIR:add_icon_settings(k,v.iDisplayName)
		if v.iEnabled then
			CIR:CreateIcon(k)
			CIR:ProcessScriptForIcon(v.script,CIR.icons[k])
		end
	end
end


local iconConfigTable=
function()
	return {
		iIcon={
			type = 'group',
			name = "General",
			order = 1,
			args={
				iEnabled={
					type='toggle',
					name='Enable',
					order=1,
					desc="Enable the icon.",
					get="getIconValue",
					set='setIconValue',
				},
				iDisplayName={
					type='input', name='Name',order=1,
					desc='Changes the name of the icon.',
					get='getIconValue',
					set='setIconValue',width='full',
				},
				iAccuracy={
					type='range', name='Amount of fractional digits',
					desc='Enables fractional digits to keep track of real Cooldowns (ex. Chain Lightning 3.5 cooldown - without it would be rounded to 3)\n0=disable, 1=1 digit after comma(,)\nCurrently only enabled if used Timeformat: 1,3 and 4\nNote: This will also lower/higher the updateFrequency.',
					min=0,max=2,
					step=1,
					get='getIconValue',
					set='setIconValue',
					disabled=true,
				},
				iZoom={
					type='toggle', name=' Toggle Zoom',--order=103,
					desc='Enables/Disables zoom for icons.',
					get="getIconValue",
					set="setIconValue",
				},
				iRange={
					type='toggle', name=' Toggle Range',--order=103,
					desc='Enables/Disables Range-Check',
					get="getIconValue",
					set="setIconValue",
				},
				iGrayscale={
					type='toggle', name=' Toggle Grayscale',--order=103,
					desc='Enables/Disables Grayscale',
					get="getIconValue",
					set="setIconValue",
				},
				iIconSize={
					type='range', name=' Icon Size',
					desc='Changes the size of icons.',
					min=2,max=124,width='full',
					get="getIconValue",
					set="setIconValue",			
				},
				iX = {
					type='range', name=' X Coordinate',
					desc='Changes the x location on screen.',
					min=-floor(UIParent:GetWidth()/2),max=floor(UIParent:GetWidth()/2),step=1,order=-2,width="double",
					get='getIconValue',
					set='setIconValue',
				},
				iY = {
					type='range', name=' Y Coordinate',
					desc='Changes the Y location on screen.',
					min=-floor(UIParent:GetHeight()/2),max=floor(UIParent:GetHeight()/2),step=1,order=-1,width="double",
					get='getIconValue',
					set='setIconValue',
				},
				iDisplayType = {
					type='select', name='Display type', order=-1,
					values = {"Show Always", "On Cooldown only", "If ready"},
					get='getIconValue',
					set='setIconValue',
				},
				iTimeFormat = {
					type='select', name='Time Format', order=-2,
					values= { "Seconds.Miliseconds", "Minute:Seconds", "Minutes", "Seconds[.Miliseconds, Miliseconds under X Seconds]", "None" },
					get='getIconValue',
					set='setIconValue',
				},
			}
		},
		iFont = {
			type = 'group',
			name = "Font",
			order = 2,
			args={
				iFontSize = {
						type='range', name=' Font size',--order=112,
						desc='Changes the size of the font.',
						min=1,max=32,width='full',
						step=1,
						get='getIconValue',
						set='setIconValue',
				},
				iFontStyle={
					type='select',name=' Font',
					values=media:List('font'), style='dropdown',
					get="getFontStyle",
					set="setFontStyle",
				},
				iFontOffsetX={
					type='range', name=' Font X-Offset',--order=112,
					desc='Changes the x-offset of the font to the center of the icon.',
					min=0,max=32,width='full',
					step=1,
					get='getIconValue',
					set='setIconValue',
				},
				iFontOffsetY={
					type='range', name=' Font Y-Offset',--order=112,
					desc='Changes the y-offset of the font to the center of the icon.',
					min=0,max=32,width='full',
					step=1,
					get='getIconValue',
					set='setIconValue',
				},
				iFontColor={
					type='color', name=' Text Color',order=2,
					desc='Changes the color of the Cooldown Text displayed on Icon.',
					hasAlpha=true,
					get='getTextColor',
					set='setTextColor',
				},
			}
		},
		iScript = {
			type='group',
			name='Script',
			order = 3,
			args = {
				script= {
					type='input',
					name='',
					width='full',
					get='getIconValue',
					set='setIconValue',
					multiline=24,
				},
			},
		},
		remCurBtn={
			type='execute',
			name='Remove current icon',
			desc='Removes the current opened icon. \n Reloading UI is highly recommended after finishing setting up new icons, removing old etc. as they all take memory space.',
			func=remCurBtn_Func,
			order=-1,
		},
	}
end
--TODO gonna add to LCI for groups too
local function GetNewID(tbl)
	--because dbs aren't sorted, create sorted ones
	local cur_id = 0
	local idt = { }
	--create id table
	for k,v in pairs(tbl) do
		idt[#idt+1] = v.id
	end
	table.sort(idt)
	for k,v in pairs(idt) do
		if v == cur_id then
			cur_id = cur_id +1
		else
			return cur_id
		end
	end
	return cur_id
end

local function addNewBtn_Func(info)
		--init value
		local db = CIR.db
		local icondb = db.char.icondb
		if not(type(icondb)=='table') then
			print("Create IconDB")
			CIR.db.char.icondb={}
		end
		local ButtonID = GetNewID(CIR.db.char.icondb)
		--ButtonID = LCI:Getn(CIR.db.char.icondb) out-dated
		--print(ButtonID)
		local newIcon = "Icon"..ButtonID
		local a_config = {
			name=newIcon,
			type='group',
			width='double',
			cmdHidden=true,
			childGroups='tab',
			args=iconConfigTable(),
		}
		local a_db = {
			iEnabled = true,
			id = ButtonID,
			script = [[ ]],
			--font
			iFontSize = db.char.gFont.gFontSize,
			iFontStyle = db.char.gFont.gFontStyle,
			iFontOffsetX = db.char.gFont.gFontOffsetX,
			iFontOffsetY = db.char.gFont.gFontOffsetY,
			iFontColor = {
				db.char.gFont.gFontColor[1],
				db.char.gFont.gFontColor[2],
				db.char.gFont.gFontColor[3],
				db.char.gFont.gFontColor[4],
			},
			--icon
			iZoom = db.char.gIcon.gZoom,
			iRange = db.char.gIcon.gRange,
			iGrayscale = db.char.gIcon.gGrayscale,
			iIconSize = db.char.gIcon.gIconSize,
			iAccuracy = db.char.gIcon.gAccuracy,
			iDisplayType = db.char.gIcon.gDisplayType,
			iTimeFormat = db.char.gIcon.gTimeFormat,
			iTimeFormat4_Border = db.char.gIcon.gTimeFormat4_Border,
			iX = 0,
			iY = 0,
			iDisplayName = newIcon,
		}
		--SV update
		CIR.db.char.icondb[newIcon] = a_db
		--add in options
		CIR.Options.args.icon_settings.args[newIcon] = a_config
		--create the frame
		CIR:CreateIcon(newIcon)
end

CIR.add_icon_settings=
function(self,icon,iDN)
	if iDN=="" then iDN = icon end
	self.Options.args.icon_settings.args[icon] = {
		name=iDN or icon,
		type='group',
		cmdHidden=true,
		width='double',
		childGroups='tab',
		args=iconConfigTable(),
	}
end

CIR.get_icon_settings=
function()
	return {
		addNewBtn={
			type='execute',
			name="Add new Icon",
			desc="Adds a new customizeable Icon for a new Cooldown to be tracked.",
			func=addNewBtn_Func,
			order=-1,
		},
	}
end

